/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.extension.controlcenter;

import org.fortiss.af3.component.model.OutputPort;
import org.fortiss.af3.expression.model.types.TBool;
import org.fortiss.af3.expression.model.types.TDouble;
import org.fortiss.af3.expression.model.types.TInt;
import org.fortiss.af3.platform.raspberry.generator.framework.IWriteableGeneratorExtension;
import org.fortiss.af3.platform.raspberry.model.controlcenter.ControlCenterUpstreamOutput;

/** Executable for controlcenter {@link ControlCenterUpstreamOutput}. */
public class ControlCenterUpstreamOutputGeneratorExtension extends
		ControlcenterGeneratorExtensionBase<ControlCenterUpstreamOutput> implements
		IWriteableGeneratorExtension {
	/** Constructor. */
	public ControlCenterUpstreamOutputGeneratorExtension(ControlCenterUpstreamOutput modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public String getWriteCode(OutputPort logicalSignal, String value) {

		// send the value depending on the logicalsiganal data type
		if(logicalSignal.getVariableType() instanceof TDouble) {
			return "af3_cc_send_double(\"" + logicalSignal.getName() + "\" , " + value +
					" , 4567);\n";
		} else if(logicalSignal.getVariableType() instanceof TInt) {
			return "af3_cc_send_int(\"" + logicalSignal.getName() + "\" , " + value + " , 4567);\n";
		} else if(logicalSignal.getVariableType() instanceof TBool) {
			return "af3_cc_send_boolean(\"" + logicalSignal.getName() + "\" , " + value +
					" , 4567);\n";
		} else {
			return "FIXME(\"Data type " + logicalSignal.getVariableType() + " of variable " +
					logicalSignal.getName() + " not supported for control centert.\");\n";
		}
	}

	/** {@inheritDoc} */
	@Override
	public String getNoValWriteCode(OutputPort logicalSignal) {
		return "af3_cc_send_noval(\"" + logicalSignal.getName() + "\" , 4567);\n";
	}
}
